# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator (autorest: 3.9.7, generator: @autorest/python@6.28.4)
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import sys
from typing import Any, AsyncIterable, Callable, Dict, IO, Optional, TypeVar, Union, cast, overload
import urllib.parse

from azure.core import AsyncPipelineClient
from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict

from ..._serialization import Deserializer, Serializer
from ...operations._operations import (
    build_participants_list_request,
    build_participants_update_request,
    build_rooms_create_request,
    build_rooms_delete_request,
    build_rooms_get_request,
    build_rooms_list_request,
    build_rooms_update_request,
)
from .._configuration import AzureCommunicationRoomsServiceConfiguration

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class RoomsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.communication.rooms.aio.AzureCommunicationRoomsService`'s
        :attr:`rooms` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AzureCommunicationRoomsServiceConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    async def create(self, create_room_request: JSON, *, content_type: str = "application/json", **kwargs: Any) -> JSON:
        """Creates a new room.

        Creates a new room.

        :param create_room_request: The create room request body. Required.
        :type create_room_request: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: JSON object
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # JSON input template you can fill out and use as your body input.
                create_room_request = {
                    "participants": {
                        "str": {
                            "role": "str"
                        }
                    },
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }

                # response body for status code(s): 201
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """

    @overload
    async def create(
        self, create_room_request: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> JSON:
        """Creates a new room.

        Creates a new room.

        :param create_room_request: The create room request body. Required.
        :type create_room_request: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: JSON object
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # response body for status code(s): 201
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """

    @distributed_trace_async
    async def create(self, create_room_request: Union[JSON, IO[bytes]], **kwargs: Any) -> JSON:
        """Creates a new room.

        Creates a new room.

        :param create_room_request: The create room request body. Is either a JSON type or a IO[bytes]
         type. Required.
        :type create_room_request: JSON or IO[bytes]
        :return: JSON object
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # JSON input template you can fill out and use as your body input.
                create_room_request = {
                    "participants": {
                        "str": {
                            "role": "str"
                        }
                    },
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }

                # response body for status code(s): 201
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[JSON] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(create_room_request, (IOBase, bytes)):
            _content = create_room_request
        else:
            _json = create_room_request

        _request = build_rooms_create_request(
            content_type=content_type,
            api_version=self._config.api_version,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if response.content:
            deserialized = response.json()
        else:
            deserialized = None

        if cls:
            return cls(pipeline_response, cast(JSON, deserialized), {})  # type: ignore

        return cast(JSON, deserialized)  # type: ignore

    @distributed_trace
    def list(self, **kwargs: Any) -> AsyncIterable[JSON]:
        """Retrieves all created rooms.

        Retrieves all created rooms.

        :return: An iterator like instance of JSON object
        :rtype: ~azure.core.async_paging.AsyncItemPaged[JSON]
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # response body for status code(s): 200
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[JSON] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_rooms_list_request(
                    api_version=self._config.api_version,
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = deserialized["value"]
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nextLink") or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def get(self, room_id: str, **kwargs: Any) -> JSON:
        """Retrieves an existing room by id.

        Retrieves an existing room by id.

        :param room_id: The id of the room requested. Required.
        :type room_id: str
        :return: JSON object
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # response body for status code(s): 200
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[JSON] = kwargs.pop("cls", None)

        _request = build_rooms_get_request(
            room_id=room_id,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if response.content:
            deserialized = response.json()
        else:
            deserialized = None

        if cls:
            return cls(pipeline_response, cast(JSON, deserialized), {})  # type: ignore

        return cast(JSON, deserialized)  # type: ignore

    @overload
    async def update(
        self,
        room_id: str,
        update_room_request: JSON,
        *,
        content_type: str = "application/merge-patch+json",
        **kwargs: Any
    ) -> JSON:
        """Update a room with given changes.

        Update a room with given changes.

        :param room_id: The id of the room requested. Required.
        :type room_id: str
        :param update_room_request: The update room request. Required.
        :type update_room_request: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/merge-patch+json".
        :paramtype content_type: str
        :return: JSON object
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # JSON input template you can fill out and use as your body input.
                update_room_request = {
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }

                # response body for status code(s): 200
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """

    @overload
    async def update(
        self,
        room_id: str,
        update_room_request: IO[bytes],
        *,
        content_type: str = "application/merge-patch+json",
        **kwargs: Any
    ) -> JSON:
        """Update a room with given changes.

        Update a room with given changes.

        :param room_id: The id of the room requested. Required.
        :type room_id: str
        :param update_room_request: The update room request. Required.
        :type update_room_request: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/merge-patch+json".
        :paramtype content_type: str
        :return: JSON object
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # response body for status code(s): 200
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """

    @distributed_trace_async
    async def update(self, room_id: str, update_room_request: Union[JSON, IO[bytes]], **kwargs: Any) -> JSON:
        """Update a room with given changes.

        Update a room with given changes.

        :param room_id: The id of the room requested. Required.
        :type room_id: str
        :param update_room_request: The update room request. Is either a JSON type or a IO[bytes] type.
         Required.
        :type update_room_request: JSON or IO[bytes]
        :return: JSON object
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # JSON input template you can fill out and use as your body input.
                update_room_request = {
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }

                # response body for status code(s): 200
                response == {
                    "createdAt": "2020-02-20 00:00:00",
                    "id": "str",
                    "pstnDialOutEnabled": bool,
                    "validFrom": "2020-02-20 00:00:00",
                    "validUntil": "2020-02-20 00:00:00"
                }
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[JSON] = kwargs.pop("cls", None)

        content_type = content_type or "application/merge-patch+json"
        _json = None
        _content = None
        if isinstance(update_room_request, (IOBase, bytes)):
            _content = update_room_request
        else:
            _json = update_room_request

        _request = build_rooms_update_request(
            room_id=room_id,
            content_type=content_type,
            api_version=self._config.api_version,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if response.content:
            deserialized = response.json()
        else:
            deserialized = None

        if cls:
            return cls(pipeline_response, cast(JSON, deserialized), {})  # type: ignore

        return cast(JSON, deserialized)  # type: ignore

    @distributed_trace_async
    async def delete(self, room_id: str, **kwargs: Any) -> None:
        """Delete a room.

        Delete a room.

        :param room_id: The id of the room to be deleted. Required.
        :type room_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_rooms_delete_request(
            room_id=room_id,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore


class ParticipantsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.communication.rooms.aio.AzureCommunicationRoomsService`'s
        :attr:`participants` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: AzureCommunicationRoomsServiceConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(self, room_id: str, **kwargs: Any) -> AsyncIterable[JSON]:
        """Get participants in a room.

        Get participants in a room.

        :param room_id: The id of the room to get participants from. Required.
        :type room_id: str
        :return: An iterator like instance of JSON object
        :rtype: ~azure.core.async_paging.AsyncItemPaged[JSON]
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # response body for status code(s): 200
                response == {
                    "rawId": "str",
                    "role": "str"
                }
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[JSON] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_participants_list_request(
                    room_id=room_id,
                    api_version=self._config.api_version,
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = deserialized["value"]
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nextLink") or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @overload
    async def update(
        self,
        room_id: str,
        update_participants_request: JSON,
        *,
        content_type: str = "application/merge-patch+json",
        **kwargs: Any
    ) -> JSON:
        """Update participants in a room.

        Update participants in a room.

        :param room_id: The id of the room to update the participants in. Required.
        :type room_id: str
        :param update_participants_request: An updated set of participants of the room. Required.
        :type update_participants_request: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/merge-patch+json".
        :paramtype content_type: str
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # JSON input template you can fill out and use as your body input.
                update_participants_request = {
                    "participants": {
                        "str": {
                            "role": "str"
                        }
                    }
                }
        """

    @overload
    async def update(
        self,
        room_id: str,
        update_participants_request: IO[bytes],
        *,
        content_type: str = "application/merge-patch+json",
        **kwargs: Any
    ) -> JSON:
        """Update participants in a room.

        Update participants in a room.

        :param room_id: The id of the room to update the participants in. Required.
        :type room_id: str
        :param update_participants_request: An updated set of participants of the room. Required.
        :type update_participants_request: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/merge-patch+json".
        :paramtype content_type: str
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def update(self, room_id: str, update_participants_request: Union[JSON, IO[bytes]], **kwargs: Any) -> JSON:
        """Update participants in a room.

        Update participants in a room.

        :param room_id: The id of the room to update the participants in. Required.
        :type room_id: str
        :param update_participants_request: An updated set of participants of the room. Is either a
         JSON type or a IO[bytes] type. Required.
        :type update_participants_request: JSON or IO[bytes]
        :return: JSON
        :rtype: JSON
        :raises ~azure.core.exceptions.HttpResponseError:

        Example:
            .. code-block:: python

                # JSON input template you can fill out and use as your body input.
                update_participants_request = {
                    "participants": {
                        "str": {
                            "role": "str"
                        }
                    }
                }
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[JSON] = kwargs.pop("cls", None)

        content_type = content_type or "application/merge-patch+json"
        _json = None
        _content = None
        if isinstance(update_participants_request, (IOBase, bytes)):
            _content = update_participants_request
        else:
            _json = update_participants_request

        _request = build_participants_update_request(
            room_id=room_id,
            content_type=content_type,
            api_version=self._config.api_version,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if response.content:
            deserialized = response.json()
        else:
            deserialized = None

        if cls:
            return cls(pipeline_response, cast(JSON, deserialized), {})  # type: ignore

        return cast(JSON, deserialized)  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import TYPE_CHECKING
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpRequest, HttpResponse

from .. import models as _models

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, Generic, Optional, TypeVar

    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

class AttestationOperations(object):
    """AttestationOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.security.attestation._generated.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    def attest_open_enclave(
        self,
        request,  # type: "_models.AttestOpenEnclaveRequest"
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.AttestationResponse"
        """Attest to an SGX enclave.

        Processes an OpenEnclave report , producing an artifact. The type of artifact produced is
        dependent upon attestation policy.

        :param request: Request object containing the quote.
        :type request: ~azure.security.attestation._generated.models.AttestOpenEnclaveRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: AttestationResponse, or the result of cls(response)
        :rtype: ~azure.security.attestation._generated.models.AttestationResponse
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.AttestationResponse"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-10-01"
        content_type = kwargs.pop("content_type", "application/json")
        accept = "application/json"

        # Construct URL
        url = self.attest_open_enclave.metadata['url']  # type: ignore
        path_format_arguments = {
            'instanceUrl': self._serialize.url("self._config.instance_url", self._config.instance_url, 'str', skip_quote=True),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        body_content_kwargs = {}  # type: Dict[str, Any]
        body_content = self._serialize.body(request, 'AttestOpenEnclaveRequest')
        body_content_kwargs['content'] = body_content
        request = self._client.post(url, query_parameters, header_parameters, **body_content_kwargs)
        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.CloudError, response)
            raise HttpResponseError(response=response, model=error)

        deserialized = self._deserialize('AttestationResponse', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    attest_open_enclave.metadata = {'url': '/attest/OpenEnclave'}  # type: ignore

    def attest_sgx_enclave(
        self,
        request,  # type: "_models.AttestSgxEnclaveRequest"
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.AttestationResponse"
        """Attest to an SGX enclave.

        Processes an SGX enclave quote, producing an artifact. The type of artifact produced is
        dependent upon attestation policy.

        :param request: Request object containing the quote.
        :type request: ~azure.security.attestation._generated.models.AttestSgxEnclaveRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: AttestationResponse, or the result of cls(response)
        :rtype: ~azure.security.attestation._generated.models.AttestationResponse
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.AttestationResponse"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        api_version = "2020-10-01"
        content_type = kwargs.pop("content_type", "application/json")
        accept = "application/json"

        # Construct URL
        url = self.attest_sgx_enclave.metadata['url']  # type: ignore
        path_format_arguments = {
            'instanceUrl': self._serialize.url("self._config.instance_url", self._config.instance_url, 'str', skip_quote=True),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        body_content_kwargs = {}  # type: Dict[str, Any]
        body_content = self._serialize.body(request, 'AttestSgxEnclaveRequest')
        body_content_kwargs['content'] = body_content
        request = self._client.post(url, query_parameters, header_parameters, **body_content_kwargs)
        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.CloudError, response)
            raise HttpResponseError(response=response, model=error)

        deserialized = self._deserialize('AttestationResponse', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    attest_sgx_enclave.metadata = {'url': '/attest/SgxEnclave'}  # type: ignore

    def attest_tpm(
        self,
        data=None,  # type: Optional[bytes]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.TpmAttestationResponse"
        """Attest a Virtualization-based Security (VBS) enclave.

        Processes attestation evidence from a VBS enclave, producing an attestation result. The
        attestation result produced is dependent upon the attestation policy.

        :param data: Protocol data containing artifacts for attestation.
        :type data: bytes
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: TpmAttestationResponse, or the result of cls(response)
        :rtype: ~azure.security.attestation._generated.models.TpmAttestationResponse
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.TpmAttestationResponse"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        _request = _models.TpmAttestationRequest(data=data)
        api_version = "2020-10-01"
        content_type = kwargs.pop("content_type", "application/json")
        accept = "application/json"

        # Construct URL
        url = self.attest_tpm.metadata['url']  # type: ignore
        path_format_arguments = {
            'instanceUrl': self._serialize.url("self._config.instance_url", self._config.instance_url, 'str', skip_quote=True),
        }
        url = self._client.format_url(url, **path_format_arguments)

        # Construct parameters
        query_parameters = {}  # type: Dict[str, Any]
        query_parameters['api-version'] = self._serialize.query("api_version", api_version, 'str')

        # Construct headers
        header_parameters = {}  # type: Dict[str, Any]
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')
        header_parameters['Accept'] = self._serialize.header("accept", accept, 'str')

        body_content_kwargs = {}  # type: Dict[str, Any]
        body_content = self._serialize.body(_request, 'TpmAttestationRequest')
        body_content_kwargs['content'] = body_content
        request = self._client.post(url, query_parameters, header_parameters, **body_content_kwargs)
        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.CloudError, response)
            raise HttpResponseError(response=response, model=error)

        deserialized = self._deserialize('TpmAttestationResponse', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized
    attest_tpm.metadata = {'url': '/attest/Tpm'}  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.compute import ComputeManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestComputeManagementVirtualMachineScaleSetVMsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ComputeManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_reimage(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_reimage(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_reimage_all(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_reimage_all(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_approve_rolling_upgrade(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_approve_rolling_upgrade(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_deallocate(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_deallocate(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_update(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_update(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            parameters={
                "location": "str",
                "additionalCapabilities": {"hibernationEnabled": bool, "ultraSSDEnabled": bool},
                "availabilitySet": {"id": "str"},
                "diagnosticsProfile": {"bootDiagnostics": {"enabled": bool, "storageUri": "str"}},
                "etag": "str",
                "hardwareProfile": {"vmSize": "str", "vmSizeProperties": {"vCPUsAvailable": 0, "vCPUsPerCore": 0}},
                "id": "str",
                "identity": {
                    "principalId": "str",
                    "tenantId": "str",
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "instanceId": "str",
                "instanceView": {
                    "assignedHost": "str",
                    "bootDiagnostics": {
                        "consoleScreenshotBlobUri": "str",
                        "serialConsoleLogBlobUri": "str",
                        "status": {
                            "code": "str",
                            "displayStatus": "str",
                            "level": "str",
                            "message": "str",
                            "time": "2020-02-20 00:00:00",
                        },
                    },
                    "computerName": "str",
                    "disks": [
                        {
                            "encryptionSettings": [
                                {
                                    "diskEncryptionKey": {"secretUrl": "str", "sourceVault": {"id": "str"}},
                                    "enabled": bool,
                                    "keyEncryptionKey": {"keyUrl": "str", "sourceVault": {"id": "str"}},
                                }
                            ],
                            "name": "str",
                            "statuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                        }
                    ],
                    "extensions": [
                        {
                            "name": "str",
                            "statuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                            "substatuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                            "type": "str",
                            "typeHandlerVersion": "str",
                        }
                    ],
                    "hyperVGeneration": "str",
                    "maintenanceRedeployStatus": {
                        "isCustomerInitiatedMaintenanceAllowed": bool,
                        "lastOperationMessage": "str",
                        "lastOperationResultCode": "str",
                        "maintenanceWindowEndTime": "2020-02-20 00:00:00",
                        "maintenanceWindowStartTime": "2020-02-20 00:00:00",
                        "preMaintenanceWindowEndTime": "2020-02-20 00:00:00",
                        "preMaintenanceWindowStartTime": "2020-02-20 00:00:00",
                    },
                    "osName": "str",
                    "osVersion": "str",
                    "placementGroupId": "str",
                    "platformFaultDomain": 0,
                    "platformUpdateDomain": 0,
                    "rdpThumbPrint": "str",
                    "statuses": [
                        {
                            "code": "str",
                            "displayStatus": "str",
                            "level": "str",
                            "message": "str",
                            "time": "2020-02-20 00:00:00",
                        }
                    ],
                    "vmAgent": {
                        "extensionHandlers": [
                            {
                                "status": {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                },
                                "type": "str",
                                "typeHandlerVersion": "str",
                            }
                        ],
                        "statuses": [
                            {
                                "code": "str",
                                "displayStatus": "str",
                                "level": "str",
                                "message": "str",
                                "time": "2020-02-20 00:00:00",
                            }
                        ],
                        "vmAgentVersion": "str",
                    },
                    "vmHealth": {
                        "status": {
                            "code": "str",
                            "displayStatus": "str",
                            "level": "str",
                            "message": "str",
                            "time": "2020-02-20 00:00:00",
                        }
                    },
                },
                "latestModelApplied": bool,
                "licenseType": "str",
                "modelDefinitionApplied": "str",
                "name": "str",
                "networkProfile": {
                    "networkApiVersion": "str",
                    "networkInterfaceConfigurations": [
                        {
                            "name": "str",
                            "auxiliaryMode": "str",
                            "auxiliarySku": "str",
                            "deleteOption": "str",
                            "disableTcpStateTracking": bool,
                            "dnsSettings": {"dnsServers": ["str"]},
                            "dscpConfiguration": {"id": "str"},
                            "enableAcceleratedNetworking": bool,
                            "enableFpga": bool,
                            "enableIPForwarding": bool,
                            "ipConfigurations": [
                                {
                                    "name": "str",
                                    "applicationGatewayBackendAddressPools": [{"id": "str"}],
                                    "applicationSecurityGroups": [{"id": "str"}],
                                    "loadBalancerBackendAddressPools": [{"id": "str"}],
                                    "primary": bool,
                                    "privateIPAddressVersion": "str",
                                    "publicIPAddressConfiguration": {
                                        "name": "str",
                                        "deleteOption": "str",
                                        "dnsSettings": {"domainNameLabel": "str", "domainNameLabelScope": "str"},
                                        "idleTimeoutInMinutes": 0,
                                        "ipTags": [{"ipTagType": "str", "tag": "str"}],
                                        "publicIPAddressVersion": "str",
                                        "publicIPAllocationMethod": "str",
                                        "publicIPPrefix": {"id": "str"},
                                        "sku": {"name": "str", "tier": "str"},
                                    },
                                    "subnet": {"id": "str"},
                                }
                            ],
                            "networkSecurityGroup": {"id": "str"},
                            "primary": bool,
                        }
                    ],
                    "networkInterfaces": [{"deleteOption": "str", "id": "str", "primary": bool}],
                },
                "networkProfileConfiguration": {
                    "networkInterfaceConfigurations": [
                        {
                            "name": "str",
                            "auxiliaryMode": "str",
                            "auxiliarySku": "str",
                            "deleteOption": "str",
                            "disableTcpStateTracking": bool,
                            "dnsSettings": {"dnsServers": ["str"]},
                            "enableAcceleratedNetworking": bool,
                            "enableFpga": bool,
                            "enableIPForwarding": bool,
                            "ipConfigurations": [
                                {
                                    "name": "str",
                                    "applicationGatewayBackendAddressPools": [{"id": "str"}],
                                    "applicationSecurityGroups": [{"id": "str"}],
                                    "loadBalancerBackendAddressPools": [{"id": "str"}],
                                    "loadBalancerInboundNatPools": [{"id": "str"}],
                                    "primary": bool,
                                    "privateIPAddressVersion": "str",
                                    "publicIPAddressConfiguration": {
                                        "name": "str",
                                        "deleteOption": "str",
                                        "dnsSettings": {"domainNameLabel": "str", "domainNameLabelScope": "str"},
                                        "idleTimeoutInMinutes": 0,
                                        "ipTags": [{"ipTagType": "str", "tag": "str"}],
                                        "publicIPAddressVersion": "str",
                                        "publicIPPrefix": {"id": "str"},
                                        "sku": {"name": "str", "tier": "str"},
                                    },
                                    "subnet": {"id": "str"},
                                }
                            ],
                            "networkSecurityGroup": {"id": "str"},
                            "primary": bool,
                        }
                    ]
                },
                "osProfile": {
                    "adminPassword": "str",
                    "adminUsername": "str",
                    "allowExtensionOperations": bool,
                    "computerName": "str",
                    "customData": "str",
                    "linuxConfiguration": {
                        "disablePasswordAuthentication": bool,
                        "enableVMAgentPlatformUpdates": bool,
                        "patchSettings": {
                            "assessmentMode": "str",
                            "automaticByPlatformSettings": {
                                "bypassPlatformSafetyChecksOnUserSchedule": bool,
                                "rebootSetting": "str",
                            },
                            "patchMode": "str",
                        },
                        "provisionVMAgent": bool,
                        "ssh": {"publicKeys": [{"keyData": "str", "path": "str"}]},
                    },
                    "requireGuestProvisionSignal": bool,
                    "secrets": [
                        {
                            "sourceVault": {"id": "str"},
                            "vaultCertificates": [{"certificateStore": "str", "certificateUrl": "str"}],
                        }
                    ],
                    "windowsConfiguration": {
                        "additionalUnattendContent": [
                            {
                                "componentName": "Microsoft-Windows-Shell-Setup",
                                "content": "str",
                                "passName": "OobeSystem",
                                "settingName": "str",
                            }
                        ],
                        "enableAutomaticUpdates": bool,
                        "enableVMAgentPlatformUpdates": bool,
                        "patchSettings": {
                            "assessmentMode": "str",
                            "automaticByPlatformSettings": {
                                "bypassPlatformSafetyChecksOnUserSchedule": bool,
                                "rebootSetting": "str",
                            },
                            "enableHotpatching": bool,
                            "patchMode": "str",
                        },
                        "provisionVMAgent": bool,
                        "timeZone": "str",
                        "winRM": {"listeners": [{"certificateUrl": "str", "protocol": "str"}]},
                    },
                },
                "plan": {"name": "str", "product": "str", "promotionCode": "str", "publisher": "str"},
                "protectionPolicy": {"protectFromScaleIn": bool, "protectFromScaleSetActions": bool},
                "provisioningState": "str",
                "resilientVMDeletionStatus": "str",
                "resources": [
                    {
                        "autoUpgradeMinorVersion": bool,
                        "enableAutomaticUpgrade": bool,
                        "forceUpdateTag": "str",
                        "id": "str",
                        "instanceView": {
                            "name": "str",
                            "statuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                            "substatuses": [
                                {
                                    "code": "str",
                                    "displayStatus": "str",
                                    "level": "str",
                                    "message": "str",
                                    "time": "2020-02-20 00:00:00",
                                }
                            ],
                            "type": "str",
                            "typeHandlerVersion": "str",
                        },
                        "location": "str",
                        "name": "str",
                        "protectedSettings": {},
                        "protectedSettingsFromKeyVault": {"secretUrl": "str", "sourceVault": {"id": "str"}},
                        "provisionAfterExtensions": ["str"],
                        "provisioningState": "str",
                        "publisher": "str",
                        "settings": {},
                        "suppressFailures": bool,
                        "tags": {"str": "str"},
                        "type": "str",
                        "typeHandlerVersion": "str",
                    }
                ],
                "securityProfile": {
                    "encryptionAtHost": bool,
                    "encryptionIdentity": {"userAssignedIdentityResourceId": "str"},
                    "proxyAgentSettings": {
                        "enabled": bool,
                        "imds": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                        "keyIncarnationId": 0,
                        "mode": "str",
                        "wireServer": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                    },
                    "securityType": "str",
                    "uefiSettings": {"secureBootEnabled": bool, "vTpmEnabled": bool},
                },
                "sku": {"capacity": 0, "name": "str", "tier": "str"},
                "storageProfile": {
                    "alignRegionalDisksToVMZone": bool,
                    "dataDisks": [
                        {
                            "createOption": "str",
                            "lun": 0,
                            "caching": "str",
                            "deleteOption": "str",
                            "detachOption": "str",
                            "diskIOPSReadWrite": 0,
                            "diskMBpsReadWrite": 0,
                            "diskSizeGB": 0,
                            "image": {"uri": "str"},
                            "managedDisk": {
                                "diskEncryptionSet": {"id": "str"},
                                "id": "str",
                                "securityProfile": {
                                    "diskEncryptionSet": {"id": "str"},
                                    "securityEncryptionType": "str",
                                },
                                "storageAccountType": "str",
                            },
                            "name": "str",
                            "sourceResource": {"id": "str"},
                            "toBeDetached": bool,
                            "vhd": {"uri": "str"},
                            "writeAcceleratorEnabled": bool,
                        }
                    ],
                    "diskControllerType": "str",
                    "imageReference": {
                        "communityGalleryImageId": "str",
                        "exactVersion": "str",
                        "id": "str",
                        "offer": "str",
                        "publisher": "str",
                        "sharedGalleryImageId": "str",
                        "sku": "str",
                        "version": "str",
                    },
                    "osDisk": {
                        "createOption": "str",
                        "caching": "str",
                        "deleteOption": "str",
                        "diffDiskSettings": {"option": "str", "placement": "str"},
                        "diskSizeGB": 0,
                        "encryptionSettings": {
                            "diskEncryptionKey": {"secretUrl": "str", "sourceVault": {"id": "str"}},
                            "enabled": bool,
                            "keyEncryptionKey": {"keyUrl": "str", "sourceVault": {"id": "str"}},
                        },
                        "image": {"uri": "str"},
                        "managedDisk": {
                            "diskEncryptionSet": {"id": "str"},
                            "id": "str",
                            "securityProfile": {"diskEncryptionSet": {"id": "str"}, "securityEncryptionType": "str"},
                            "storageAccountType": "str",
                        },
                        "name": "str",
                        "osType": "str",
                        "vhd": {"uri": "str"},
                        "writeAcceleratorEnabled": bool,
                    },
                },
                "tags": {"str": "str"},
                "timeCreated": "2020-02-20 00:00:00",
                "type": "str",
                "userData": "str",
                "vmId": "str",
                "zones": ["str"],
            },
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_delete(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_delete(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_get(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.get(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_get_instance_view(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.get_instance_view(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_list(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.list(
            resource_group_name=resource_group.name,
            virtual_machine_scale_set_name="str",
            api_version="2024-11-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_power_off(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_power_off(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_restart(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_restart(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_start(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_start(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_redeploy(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_redeploy(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_retrieve_boot_diagnostics_data(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.retrieve_boot_diagnostics_data(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_perform_maintenance(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_perform_maintenance(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_simulate_eviction(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.simulate_eviction(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_attach_detach_data_disks(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_attach_detach_data_disks(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            parameters={
                "dataDisksToAttach": [
                    {
                        "diskId": "str",
                        "caching": "str",
                        "deleteOption": "str",
                        "diskEncryptionSet": {"id": "str"},
                        "lun": 0,
                        "writeAcceleratorEnabled": bool,
                    }
                ],
                "dataDisksToDetach": [{"diskId": "str", "detachOption": "str"}],
            },
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_virtual_machine_scale_set_vms_begin_run_command(self, resource_group):
        response = self.client.virtual_machine_scale_set_vms.begin_run_command(
            resource_group_name=resource_group.name,
            vm_scale_set_name="str",
            instance_id="str",
            parameters={"commandId": "str", "parameters": [{"name": "str", "value": "str"}], "script": ["str"]},
            api_version="2024-11-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# pylint: disable=too-many-lines
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import json
import os
import azure.ai.inference as sdk
from azure.ai.inference.tracing import AIInferenceInstrumentor

from model_inference_test_base import (
    ModelClientTestBase,
    ServicePreparerChatCompletions,
)
from azure.core.settings import settings
from devtools_testutils import recorded_by_proxy
from memory_trace_exporter import MemoryTraceExporter
from gen_ai_trace_verifier import GenAiTraceVerifier
from opentelemetry import trace
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import SimpleSpanProcessor

CONTENT_TRACING_ENV_VARIABLE = "AZURE_TRACING_GEN_AI_CONTENT_RECORDING_ENABLED"
content_tracing_initial_value = os.getenv(CONTENT_TRACING_ENV_VARIABLE)


# The test class name needs to start with "Test" to get collected by pytest
class TestClientTracing(ModelClientTestBase):

    @classmethod
    def teardown_class(cls):
        if content_tracing_initial_value is not None:
            os.environ[CONTENT_TRACING_ENV_VARIABLE] = content_tracing_initial_value

    # **********************************************************************************
    #
    #                            TRACING TESTS - CHAT COMPLETIONS
    #
    # **********************************************************************************

    def setup_memory_trace_exporter(self) -> MemoryTraceExporter:
        # Setup Azure Core settings to use OpenTelemetry tracing
        settings.tracing_implementation = "OpenTelemetry"
        trace.set_tracer_provider(TracerProvider())
        tracer = trace.get_tracer(__name__)
        memoryExporter = MemoryTraceExporter()
        span_processor = SimpleSpanProcessor(memoryExporter)
        trace.get_tracer_provider().add_span_processor(span_processor)
        return span_processor, memoryExporter

    def modify_env_var(self, name, new_value):
        current_value = os.getenv(name)
        os.environ[name] = new_value
        return current_value

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_instrumentation(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        client = self._create_chat_client(**kwargs)
        exception_caught = False
        try:
            assert AIInferenceInstrumentor().is_instrumented() == False
            AIInferenceInstrumentor().instrument()
            assert AIInferenceInstrumentor().is_instrumented() == True
            AIInferenceInstrumentor().uninstrument()
            assert AIInferenceInstrumentor().is_instrumented() == False
        except RuntimeError as e:
            exception_caught = True
            print(e)
        client.close()
        assert exception_caught == False

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_instrumenting_twice_does_not_cause_exception(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        client = self._create_chat_client(**kwargs)
        exception_caught = False
        try:
            AIInferenceInstrumentor().instrument()
            AIInferenceInstrumentor().instrument()
        except RuntimeError as e:
            exception_caught = True
            print(e)
        AIInferenceInstrumentor().uninstrument()
        client.close()
        assert exception_caught == False

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_uninstrumenting_uninstrumented_does_not_cause_exception(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        client = self._create_chat_client(**kwargs)
        exception_caught = False
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            exception_caught = True
            print(e)
        client.close()
        assert exception_caught == False

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_uninstrumenting_twice_does_not_cause_exception(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        client = self._create_chat_client(**kwargs)
        exception_caught = False
        uninstrumented_once = False
        try:
            AIInferenceInstrumentor().instrument()
            AIInferenceInstrumentor().uninstrument()
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            exception_caught = True
            print(e)
        client.close()
        assert exception_caught == False

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_is_content_recording_enabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
        client = self._create_chat_client(**kwargs)
        exception_caught = False
        uninstrumented_once = False
        try:
            # From environment variable instrumenting from uninstrumented
            AIInferenceInstrumentor().instrument()
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
            AIInferenceInstrumentor().instrument()
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == False
            AIInferenceInstrumentor().uninstrument()
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
            AIInferenceInstrumentor().instrument()
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == True
            AIInferenceInstrumentor().uninstrument()
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "invalid")
            AIInferenceInstrumentor().instrument()
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == False

            # From environment variable instrumenting from instrumented
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
            AIInferenceInstrumentor().instrument()
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == True
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
            AIInferenceInstrumentor().instrument()
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == True
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "invalid")
            AIInferenceInstrumentor().instrument()
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == False

            # From parameter instrumenting from uninstrumented
            AIInferenceInstrumentor().uninstrument()
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
            AIInferenceInstrumentor().instrument(enable_content_recording=False)
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == False
            AIInferenceInstrumentor().uninstrument()
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
            AIInferenceInstrumentor().instrument(enable_content_recording=True)
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == True

            # From parameter instrumenting from instrumented
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
            AIInferenceInstrumentor().instrument(enable_content_recording=False)
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == False
            self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
            AIInferenceInstrumentor().instrument(enable_content_recording=True)
            content_recording_enabled = AIInferenceInstrumentor().is_content_recording_enabled()
            assert content_recording_enabled == True
        except RuntimeError as e:
            exception_caught = True
            print(e)
        client.close()
        assert exception_caught == False

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_tracing_content_recording_disabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()
        response = client.complete(
            messages=[
                sdk.models.SystemMessage(content="You are a helpful assistant."),
                sdk.models.UserMessage(content="What is the capital of France?"),
            ],
        )
        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat ")
        if len(spans) == 0:
            spans = exporter.get_spans_by_name("chat")
        assert len(spans) == 1
        span = spans[0]
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(span, expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.choice",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"finish_reason": "stop", "index": 0}',
                },
            }
        ]
        events_match = GenAiTraceVerifier().check_span_events(span, expected_events)
        assert events_match == True
        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_tracing_content_recording_enabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()
        response = client.complete(
            messages=[
                sdk.models.SystemMessage(content="You are a helpful assistant."),
                sdk.models.UserMessage(content="What is the capital of France?"),
            ],
        )
        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat ")
        if len(spans) == 0:
            spans = exporter.get_spans_by_name("chat")
        assert len(spans) == 1
        span = spans[0]
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(span, expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.system.message",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "system", "content": "You are a helpful assistant."}',
                },
            },
            {
                "name": "gen_ai.user.message",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "user", "content": "What is the capital of France?"}',
                },
            },
            {
                "name": "gen_ai.choice",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"message": {"content": "*"}, "finish_reason": "stop", "index": 0}',
                },
            },
        ]
        events_match = GenAiTraceVerifier().check_span_events(span, expected_events)
        assert events_match == True
        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_tracing_content_unicode(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
        client = self._create_chat_client(**kwargs)
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()
        response = client.complete(
            messages=[
                sdk.models.SystemMessage(content="You are a helpful assistant."),
                sdk.models.UserMessage(content="将“hello world”翻译成中文和乌克兰语"),
            ],
        )
        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat")
        assert len(spans) == 1
        expected_events = [
            {
                "name": "gen_ai.system.message",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "system", "content": "You are a helpful assistant."}',
                },
            },
            {
                "name": "gen_ai.user.message",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "user", "content": "将“hello world”翻译成中文和乌克兰语"}',
                },
            },
            {
                "name": "gen_ai.choice",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"message": {"content": "*"}, "finish_reason": "stop", "index": 0}',
                },
            },
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[0], expected_events)
        assert events_match == True

        completion_event_content = json.loads(spans[0].events[2].attributes["gen_ai.event.content"])
        assert False == completion_event_content["message"]["content"].isascii()
        assert response.choices[0].message.content == completion_event_content["message"]["content"]
        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_streaming_tracing_content_recording_disabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()
        response = client.complete(
            messages=[
                sdk.models.SystemMessage(content="You are a helpful assistant."),
                sdk.models.UserMessage(content="What is the capital of France?"),
            ],
            stream=True,
        )
        response_content = ""
        for update in response:
            if update.choices and update.choices[0].delta.content:
                response_content = response_content + update.choices[0].delta.content
        client.close()

        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat")
        assert len(spans) == 1
        span = spans[0]
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(span, expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.choice",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"finish_reason": "stop", "index": 0}',
                },
            }
        ]
        events_match = GenAiTraceVerifier().check_span_events(span, expected_events)
        assert events_match == True
        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_streaming_tracing_content_recording_enabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()
        response = client.complete(
            messages=[
                sdk.models.SystemMessage(content="You are a helpful assistant."),
                sdk.models.UserMessage(content="What is the capital of France?"),
            ],
            stream=True,
        )
        response_content = ""
        for update in response:
            if update.choices and update.choices[0].delta.content:
                response_content = response_content + update.choices[0].delta.content
        client.close()

        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat ")
        if len(spans) == 0:
            spans = exporter.get_spans_by_name("chat")
        assert len(spans) == 1
        span = spans[0]
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(span, expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.system.message",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "system", "content": "You are a helpful assistant."}',
                },
            },
            {
                "name": "gen_ai.user.message",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "user", "content": "What is the capital of France?"}',
                },
            },
            {
                "name": "gen_ai.choice",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"message": {"content": "*"}, "finish_reason": "stop", "index": 0}',
                },
            },
        ]
        events_match = GenAiTraceVerifier().check_span_events(span, expected_events)
        assert events_match == True
        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_with_function_call_tracing_content_recording_enabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        import json
        from azure.ai.inference.models import (
            CompletionsFinishReason,
            ToolMessage,
            AssistantMessage,
            ChatCompletionsToolCall,
            ChatCompletionsToolDefinition,
            FunctionDefinition,
        )
        from azure.ai.inference import ChatCompletionsClient

        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()

        def get_weather(city: str) -> str:
            if city == "Seattle":
                return "Nice weather"
            elif city == "New York City":
                return "Good weather"
            else:
                return "Unavailable"

        weather_description = ChatCompletionsToolDefinition(
            function=FunctionDefinition(
                name="get_weather",
                description="Returns description of the weather in the specified city",
                parameters={
                    "type": "object",
                    "properties": {
                        "city": {
                            "type": "string",
                            "description": "The name of the city for which weather info is requested",
                        },
                    },
                    "required": ["city"],
                },
            )
        )
        messages = [
            sdk.models.SystemMessage(content="You are a helpful assistant."),
            sdk.models.UserMessage(content="What is the weather in Seattle?"),
        ]

        response = client.complete(messages=messages, tools=[weather_description])

        if response.choices[0].finish_reason == CompletionsFinishReason.TOOL_CALLS:
            # Append the previous model response to the chat history
            messages.append(AssistantMessage(tool_calls=response.choices[0].message.tool_calls))
            # The tool should be of type function call.
            if response.choices[0].message.tool_calls is not None and len(response.choices[0].message.tool_calls) > 0:
                for tool_call in response.choices[0].message.tool_calls:
                    if type(tool_call) is ChatCompletionsToolCall:
                        function_args = json.loads(tool_call.function.arguments.replace("'", '"'))
                        print(f"Calling function `{tool_call.function.name}` with arguments {function_args}")
                        callable_func = locals()[tool_call.function.name]
                        function_response = callable_func(**function_args)
                        print(f"Function response = {function_response}")
                        # Provide the tool response to the model, by appending it to the chat history
                        messages.append(ToolMessage(tool_call_id=tool_call.id, content=function_response))
                # With the additional tools information on hand, get another response from the model
                response = client.complete(messages=messages, tools=[weather_description])
        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat")
        assert len(spans) == 2
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("tool_calls",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[0], expected_attributes)
        assert attributes_match == True
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[1], expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.system.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "system", "content": "You are a helpful assistant."}',
                },
            },
            {
                "name": "gen_ai.user.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "user", "content": "What is the weather in Seattle?"}',
                },
            },
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"message": {"content": "", "tool_calls": [{"function": {"arguments": "{\\"city\\":\\"Seattle\\"}", "call_id": null, "name": "get_weather"}, "id": "*", "type": "function"}]}, "finish_reason": "tool_calls", "index": 0}',
                },
            },
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[0], expected_events)
        assert events_match == True

        expected_events = [
            {
                "name": "gen_ai.system.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "system", "content": "You are a helpful assistant."}',
                },
            },
            {
                "name": "gen_ai.user.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "user", "content": "What is the weather in Seattle?"}',
                },
            },
            {
                "name": "gen_ai.assistant.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "assistant", "tool_calls": [{"function": {"arguments": "{\\"city\\": \\"Seattle\\"}", "call_id": null, "name": "get_weather"}, "id": "*", "type": "function"}]}',
                },
            },
            {
                "name": "gen_ai.tool.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "tool", "tool_call_id": "*", "content": "Nice weather"}',
                },
            },
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"message": {"content": "*"}, "finish_reason": "stop", "index": 0}',
                },
            },
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[1], expected_events)
        assert events_match == True

        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_with_function_call_tracing_content_recording_disabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        import json
        from azure.ai.inference.models import (
            CompletionsFinishReason,
            ToolMessage,
            AssistantMessage,
            ChatCompletionsToolCall,
            ChatCompletionsToolDefinition,
            FunctionDefinition,
        )
        from azure.ai.inference import ChatCompletionsClient

        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()

        def get_weather(city: str) -> str:
            if city == "Seattle":
                return "Nice weather"
            elif city == "New York City":
                return "Good weather"
            else:
                return "Unavailable"

        weather_description = ChatCompletionsToolDefinition(
            function=FunctionDefinition(
                name="get_weather",
                description="Returns description of the weather in the specified city",
                parameters={
                    "type": "object",
                    "properties": {
                        "city": {
                            "type": "string",
                            "description": "The name of the city for which weather info is requested",
                        },
                    },
                    "required": ["city"],
                },
            )
        )
        messages = [
            sdk.models.SystemMessage(content="You are a helpful assistant."),
            sdk.models.UserMessage(content="What is the weather in Seattle?"),
        ]

        response = client.complete(messages=messages, tools=[weather_description])

        if response.choices[0].finish_reason == CompletionsFinishReason.TOOL_CALLS:
            # Append the previous model response to the chat history
            messages.append(AssistantMessage(tool_calls=response.choices[0].message.tool_calls))
            # The tool should be of type function call.
            if response.choices[0].message.tool_calls is not None and len(response.choices[0].message.tool_calls) > 0:
                for tool_call in response.choices[0].message.tool_calls:
                    if type(tool_call) is ChatCompletionsToolCall:
                        function_args = json.loads(tool_call.function.arguments.replace("'", '"'))
                        print(f"Calling function `{tool_call.function.name}` with arguments {function_args}")
                        callable_func = locals()[tool_call.function.name]
                        function_response = callable_func(**function_args)
                        print(f"Function response = {function_response}")
                        # Provide the tool response to the model, by appending it to the chat history
                        messages.append(ToolMessage(tool_call_id=tool_call.id, content=function_response))
                # With the additional tools information on hand, get another response from the model
                response = client.complete(messages=messages, tools=[weather_description])
        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat ")
        if len(spans) == 0:
            spans = exporter.get_spans_by_name("chat")
        assert len(spans) == 2
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("tool_calls",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[0], expected_attributes)
        assert attributes_match == True
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[1], expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"finish_reason": "tool_calls", "index": 0, "message": {"tool_calls": [{"function": {"call_id": null}, "id": "*", "type": "function"}]}}',
                },
            }
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[0], expected_events)
        assert events_match == True

        expected_events = [
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"finish_reason": "stop", "index": 0}',
                },
            }
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[1], expected_events)
        assert events_match == True

        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_with_function_call_streaming_tracing_content_recording_enabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        import json
        from azure.ai.inference.models import (
            FunctionCall,
            ToolMessage,
            AssistantMessage,
            ChatCompletionsToolCall,
            ChatCompletionsToolDefinition,
            FunctionDefinition,
        )
        from azure.ai.inference import ChatCompletionsClient

        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "True")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()

        def get_weather(city: str) -> str:
            if city == "Seattle":
                return "Nice weather"
            elif city == "New York City":
                return "Good weather"
            else:
                return "Unavailable"

        weather_description = ChatCompletionsToolDefinition(
            function=FunctionDefinition(
                name="get_weather",
                description="Returns description of the weather in the specified city",
                parameters={
                    "type": "object",
                    "properties": {
                        "city": {
                            "type": "string",
                            "description": "The name of the city for which weather info is requested",
                        },
                    },
                    "required": ["city"],
                },
            )
        )
        messages = [
            sdk.models.SystemMessage(content="You are a helpful AI assistant."),
            sdk.models.UserMessage(content="What is the weather in Seattle?"),
        ]

        response = client.complete(messages=messages, tools=[weather_description], stream=True)

        # At this point we expect a function tool call in the model response
        tool_call_id: str = ""
        function_name: str = ""
        function_args: str = ""
        for update in response:
            if update.choices[0].delta.tool_calls is not None:
                if update.choices[0].delta.tool_calls[0].function.name is not None:
                    function_name = update.choices[0].delta.tool_calls[0].function.name
                if update.choices[0].delta.tool_calls[0].id is not None:
                    tool_call_id = update.choices[0].delta.tool_calls[0].id
                function_args += update.choices[0].delta.tool_calls[0].function.arguments or ""

        # Append the previous model response to the chat history
        messages.append(
            AssistantMessage(
                tool_calls=[
                    ChatCompletionsToolCall(
                        id=tool_call_id, function=FunctionCall(name=function_name, arguments=function_args)
                    )
                ]
            )
        )

        # Make the function call
        callable_func = locals()[function_name]
        function_args_mapping = json.loads(function_args.replace("'", '"'))
        function_response = callable_func(**function_args_mapping)

        # Append the function response as a tool message to the chat history
        messages.append(ToolMessage(tool_call_id=tool_call_id, content=function_response))

        # With the additional tools information on hand, get another streaming response from the model
        response = client.complete(messages=messages, tools=[weather_description], stream=True)

        content = ""
        for update in response:
            content = content + update.choices[0].delta.content

        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat ")
        if len(spans) == 0:
            spans = exporter.get_spans_by_name("chat")
        assert len(spans) == 2
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("tool_calls",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[0], expected_attributes)
        assert attributes_match == True
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[1], expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.system.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "system", "content": "You are a helpful AI assistant."}',
                },
            },
            {
                "name": "gen_ai.user.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "user", "content": "What is the weather in Seattle?"}',
                },
            },
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"finish_reason": "tool_calls", "message": {"tool_calls": [{"id": "*", "type": "function", "function": {"name": "get_weather", "arguments": "{\\"city\\": \\"Seattle\\"}"}}]}, "index": 0}',
                },
            },
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[0], expected_events)
        assert events_match == True

        expected_events = [
            {
                "name": "gen_ai.system.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "system", "content": "You are a helpful AI assistant."}',
                },
            },
            {
                "name": "gen_ai.user.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "user", "content": "What is the weather in Seattle?"}',
                },
            },
            {
                "name": "gen_ai.assistant.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "assistant", "tool_calls": [{"id": "*", "function": {"name": "get_weather", "arguments": "{\\"city\\": \\"Seattle\\"}"}, "type": "function"}]}',
                },
            },
            {
                "name": "gen_ai.tool.message",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"role": "tool", "tool_call_id": "*", "content": "Nice weather"}',
                },
            },
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"message": {"content": "*"}, "finish_reason": "stop", "index": 0}',
                },
            },
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[1], expected_events)
        assert events_match == True

        AIInferenceInstrumentor().uninstrument()

    @ServicePreparerChatCompletions()
    @recorded_by_proxy
    def test_chat_completion_with_function_call_streaming_tracing_content_recording_disabled(self, **kwargs):
        # Make sure code is not instrumented due to a previous test exception
        try:
            AIInferenceInstrumentor().uninstrument()
        except RuntimeError as e:
            pass
        import json
        from azure.ai.inference.models import (
            FunctionCall,
            ToolMessage,
            AssistantMessage,
            ChatCompletionsToolCall,
            ChatCompletionsToolDefinition,
            FunctionDefinition,
        )
        from azure.ai.inference import ChatCompletionsClient

        self.modify_env_var(CONTENT_TRACING_ENV_VARIABLE, "False")
        client = self._create_chat_client(**kwargs)
        model = kwargs.pop("azure_ai_chat_model").lower()
        processor, exporter = self.setup_memory_trace_exporter()
        AIInferenceInstrumentor().instrument()

        def get_weather(city: str) -> str:
            if city == "Seattle":
                return "Nice weather"
            elif city == "New York City":
                return "Good weather"
            else:
                return "Unavailable"

        weather_description = ChatCompletionsToolDefinition(
            function=FunctionDefinition(
                name="get_weather",
                description="Returns description of the weather in the specified city",
                parameters={
                    "type": "object",
                    "properties": {
                        "city": {
                            "type": "string",
                            "description": "The name of the city for which weather info is requested",
                        },
                    },
                    "required": ["city"],
                },
            )
        )
        messages = [
            sdk.models.SystemMessage(content="You are a helpful assistant."),
            sdk.models.UserMessage(content="What is the weather in Seattle?"),
        ]

        response = client.complete(messages=messages, tools=[weather_description], stream=True)

        # At this point we expect a function tool call in the model response
        tool_call_id: str = ""
        function_name: str = ""
        function_args: str = ""
        for update in response:
            if update.choices[0].delta.tool_calls is not None:
                if update.choices[0].delta.tool_calls[0].function.name is not None:
                    function_name = update.choices[0].delta.tool_calls[0].function.name
                if update.choices[0].delta.tool_calls[0].id is not None:
                    tool_call_id = update.choices[0].delta.tool_calls[0].id
                function_args += update.choices[0].delta.tool_calls[0].function.arguments or ""

        # Append the previous model response to the chat history
        messages.append(
            AssistantMessage(
                tool_calls=[
                    ChatCompletionsToolCall(
                        id=tool_call_id, function=FunctionCall(name=function_name, arguments=function_args)
                    )
                ]
            )
        )

        # Make the function call
        callable_func = locals()[function_name]
        function_args_mapping = json.loads(function_args.replace("'", '"'))
        function_response = callable_func(**function_args_mapping)

        # Append the function response as a tool message to the chat history
        messages.append(ToolMessage(tool_call_id=tool_call_id, content=function_response))

        # With the additional tools information on hand, get another streaming response from the model
        response = client.complete(messages=messages, tools=[weather_description], stream=True)

        content = ""
        for update in response:
            content = content + update.choices[0].delta.content

        processor.force_flush()
        spans = exporter.get_spans_by_name_starts_with("chat ")
        if len(spans) == 0:
            spans = exporter.get_spans_by_name("chat")
        assert len(spans) == 2
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("tool_calls",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[0], expected_attributes)
        assert attributes_match == True
        expected_attributes = [
            ("gen_ai.operation.name", "chat"),
            ("gen_ai.system", "az.ai.inference"),
            ("gen_ai.request.model", "chat"),
            ("server.address", ""),
            ("gen_ai.response.id", ""),
            ("gen_ai.response.model", model),
            ("gen_ai.usage.input_tokens", "+"),
            ("gen_ai.usage.output_tokens", "+"),
            ("gen_ai.response.finish_reasons", ("stop",)),
        ]
        attributes_match = GenAiTraceVerifier().check_span_attributes(spans[1], expected_attributes)
        assert attributes_match == True

        expected_events = [
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"finish_reason": "tool_calls", "message": {"tool_calls": [{"id": "*", "type": "function"}]}, "index": 0}',
                },
            }
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[0], expected_events)
        assert events_match == True

        expected_events = [
            {
                "name": "gen_ai.choice",
                "timestamp": "*",
                "attributes": {
                    "gen_ai.system": "az.ai.inference",
                    "gen_ai.event.content": '{"finish_reason": "stop", "index": 0}',
                },
            }
        ]
        events_match = GenAiTraceVerifier().check_span_events(spans[1], expected_events)
        assert events_match == True

        AIInferenceInstrumentor().uninstrument()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.automanage import AutomanageClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-automanage
# USAGE
    python update_configuration_profile.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AutomanageClient(
        credential=DefaultAzureCredential(),
        subscription_id="mySubscriptionId",
    )

    response = client.configuration_profiles.update(
        configuration_profile_name="customConfigurationProfile",
        resource_group_name="myResourceGroupName",
        parameters={
            "properties": {
                "configuration": {
                    "Antimalware/Enable": False,
                    "AzureSecurityCenter/Enable": True,
                    "Backup/Enable": False,
                    "BootDiagnostics/Enable": True,
                    "ChangeTrackingAndInventory/Enable": True,
                    "GuestConfiguration/Enable": True,
                    "LogAnalytics/Enable": True,
                    "UpdateManagement/Enable": True,
                    "VMInsights/Enable": True,
                }
            },
            "tags": {"Organization": "Administration"},
        },
    )
    print(response)


# x-ms-original-file: specification/automanage/resource-manager/Microsoft.Automanage/stable/2022-05-04/examples/updateConfigurationProfile.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.azurearcdata import AzureArcDataManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-azurearcdata
# USAGE
    python create_or_update_sql_managed_instance.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AzureArcDataManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-1111-2222-3333-444444444444",
    )

    response = client.sql_managed_instances.begin_create(
        resource_group_name="testrg",
        sql_managed_instance_name="testsqlManagedInstance",
        sql_managed_instance={
            "extendedLocation": {
                "name": "/subscriptions/00000000-1111-2222-3333-444444444444/resourceGroups/testrg/providers/Microsoft.ExtendedLocation/customLocations/arclocation",
                "type": "CustomLocation",
            },
            "location": "northeurope",
            "properties": {
                "activeDirectoryInformation": {"keytabInformation": {"keytab": "********"}},
                "admin": "Admin user",
                "basicLoginInformation": {"password": "********", "username": "username"},
                "clusterId": "/subscriptions/00000000-1111-2222-3333-444444444444/resourceGroups/testrg/providers/Microsoft.Kubernetes/connectedClusters/connectedk8s",
                "endTime": "Instance end time",
                "extensionId": "/subscriptions/00000000-1111-2222-3333-444444444444/resourceGroups/testrg/providers/Microsoft.Kubernetes/connectedClusters/connectedk8s/providers/Microsoft.KubernetesConfiguration/extensions/extension",
                "k8sRaw": {
                    "additionalProperty": 1234,
                    "spec": {
                        "replicas": 1,
                        "scheduling": {
                            "default": {
                                "resources": {
                                    "limits": {"additionalProperty": "additionalValue", "cpu": "1", "memory": "8Gi"},
                                    "requests": {"additionalProperty": "additionalValue", "cpu": "1", "memory": "8Gi"},
                                }
                            }
                        },
                    },
                },
                "licenseType": "LicenseIncluded",
                "startTime": "Instance start time",
            },
            "sku": {"dev": True, "name": "vCore", "tier": "GeneralPurpose"},
            "tags": {"mytag": "myval"},
        },
    ).result()
    print(response)


# x-ms-original-file: specification/azurearcdata/resource-manager/Microsoft.AzureArcData/preview/2022-03-01-preview/examples/CreateOrUpdateSqlManagedInstance.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhci import AzureStackHCIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIExtensionsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_list_by_arc_setting(self, resource_group):
        response = self.client.extensions.list_by_arc_setting(
            resource_group_name=resource_group.name,
            cluster_name="str",
            arc_setting_name="str",
            api_version="2024-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_get(self, resource_group):
        response = self.client.extensions.get(
            resource_group_name=resource_group.name,
            cluster_name="str",
            arc_setting_name="str",
            extension_name="str",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_create(self, resource_group):
        response = self.client.extensions.begin_create(
            resource_group_name=resource_group.name,
            cluster_name="str",
            arc_setting_name="str",
            extension_name="str",
            extension={
                "aggregateState": "str",
                "autoUpgradeMinorVersion": bool,
                "enableAutomaticUpgrade": bool,
                "forceUpdateTag": "str",
                "id": "str",
                "managedBy": "str",
                "name": "str",
                "perNodeExtensionDetails": [
                    {
                        "extension": "str",
                        "instanceView": {
                            "name": "str",
                            "status": {
                                "code": "str",
                                "displayStatus": "str",
                                "level": "str",
                                "message": "str",
                                "time": "2020-02-20 00:00:00",
                            },
                            "type": "str",
                            "typeHandlerVersion": "str",
                        },
                        "name": "str",
                        "state": "str",
                        "typeHandlerVersion": "str",
                    }
                ],
                "protectedSettings": {},
                "provisioningState": "str",
                "publisher": "str",
                "settings": {},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
                "typeHandlerVersion": "str",
            },
            api_version="2024-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_update(self, resource_group):
        response = self.client.extensions.begin_update(
            resource_group_name=resource_group.name,
            cluster_name="str",
            arc_setting_name="str",
            extension_name="str",
            extension={
                "extensionParameters": {
                    "enableAutomaticUpgrade": bool,
                    "protectedSettings": {},
                    "settings": {},
                    "typeHandlerVersion": "str",
                }
            },
            api_version="2024-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_delete(self, resource_group):
        response = self.client.extensions.begin_delete(
            resource_group_name=resource_group.name,
            cluster_name="str",
            arc_setting_name="str",
            extension_name="str",
            api_version="2024-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_upgrade(self, resource_group):
        response = self.client.extensions.begin_upgrade(
            resource_group_name=resource_group.name,
            cluster_name="str",
            arc_setting_name="str",
            extension_name="str",
            extension_upgrade_parameters={"targetVersion": "str"},
            api_version="2024-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.costmanagement import CostManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-costmanagement
# USAGE
    python export_create_or_update_by_department.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = CostManagementClient(
        credential=DefaultAzureCredential(),
    )

    response = client.exports.create_or_update(
        scope="providers/Microsoft.Billing/billingAccounts/12/departments/1234",
        export_name="TestExport",
        parameters={
            "properties": {
                "definition": {
                    "dataSet": {
                        "configuration": {"columns": ["Date", "MeterId", "ResourceId", "ResourceLocation", "Quantity"]},
                        "granularity": "Daily",
                    },
                    "timeframe": "MonthToDate",
                    "type": "ActualCost",
                },
                "deliveryInfo": {
                    "destination": {
                        "container": "exports",
                        "resourceId": "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/MYDEVTESTRG/providers/Microsoft.Storage/storageAccounts/ccmeastusdiag182",
                        "rootFolderPath": "ad-hoc",
                    }
                },
                "format": "Csv",
                "schedule": {
                    "recurrence": "Weekly",
                    "recurrencePeriod": {"from": "2020-06-01T00:00:00Z", "to": "2020-10-31T00:00:00Z"},
                    "status": "Active",
                },
            }
        },
    )
    print(response)


# x-ms-original-file: specification/cost-management/resource-manager/Microsoft.CostManagement/stable/2022-10-01/examples/ExportCreateOrUpdateByDepartment.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.datafactory import DataFactoryManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-datafactory
# USAGE
    python pipelines_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DataFactoryManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="12345678-1234-1234-1234-12345678abc",
    )

    response = client.pipelines.create_or_update(
        resource_group_name="exampleResourceGroup",
        factory_name="exampleFactoryName",
        pipeline_name="examplePipeline",
        pipeline={
            "properties": {
                "activities": [
                    {
                        "name": "ExampleForeachActivity",
                        "type": "ForEach",
                        "typeProperties": {
                            "activities": [
                                {
                                    "inputs": [
                                        {
                                            "parameters": {
                                                "MyFileName": "examplecontainer.csv",
                                                "MyFolderPath": "examplecontainer",
                                            },
                                            "referenceName": "exampleDataset",
                                            "type": "DatasetReference",
                                        }
                                    ],
                                    "name": "ExampleCopyActivity",
                                    "outputs": [
                                        {
                                            "parameters": {
                                                "MyFileName": {"type": "Expression", "value": "@item()"},
                                                "MyFolderPath": "examplecontainer",
                                            },
                                            "referenceName": "exampleDataset",
                                            "type": "DatasetReference",
                                        }
                                    ],
                                    "type": "Copy",
                                    "typeProperties": {
                                        "dataIntegrationUnits": 32,
                                        "sink": {"type": "BlobSink"},
                                        "source": {"type": "BlobSource"},
                                    },
                                }
                            ],
                            "isSequential": True,
                            "items": {"type": "Expression", "value": "@pipeline().parameters.OutputBlobNameList"},
                        },
                    }
                ],
                "description": "Example description",
                "parameters": {"OutputBlobNameList": {"type": "Array"}},
                "policy": {"elapsedTimeMetric": {"duration": "0.00:10:00"}},
            }
        },
    )
    print(response)


# x-ms-original-file: specification/datafactory/resource-manager/Microsoft.DataFactory/stable/2018-06-01/examples/Pipelines_Update.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential
from azure.mgmt.consumption import ConsumptionManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-consumption
# USAGE
    python create_or_update_budget.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ConsumptionManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-0000-0000-0000-000000000000",
    )

    response = client.budgets.create_or_update(
        scope="subscriptions/00000000-0000-0000-0000-000000000000",
        budget_name="TestBudget",
        parameters={
            "eTag": '"1d34d016a593709"',
            "properties": {
                "amount": 100.65,
                "category": "Cost",
                "filter": {
                    "and": [
                        {
                            "dimensions": {
                                "name": "ResourceId",
                                "operator": "In",
                                "values": [
                                    "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/MYDEVTESTRG/providers/Microsoft.Compute/virtualMachines/MSVM2",
                                    "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/MYDEVTESTRG/providers/Microsoft.Compute/virtualMachines/platformcloudplatformGeneric1",
                                ],
                            }
                        },
                        {"tags": {"name": "category", "operator": "In", "values": ["Dev", "Prod"]}},
                        {"tags": {"name": "department", "operator": "In", "values": ["engineering", "sales"]}},
                    ]
                },
                "notifications": {
                    "Actual_GreaterThan_80_Percent": {
                        "contactEmails": ["johndoe@contoso.com", "janesmith@contoso.com"],
                        "contactGroups": [
                            "/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/MYDEVTESTRG/providers/microsoft.insights/actionGroups/SampleActionGroup"
                        ],
                        "contactRoles": ["Contributor", "Reader"],
                        "enabled": True,
                        "locale": "en-us",
                        "operator": "GreaterThan",
                        "threshold": 80,
                        "thresholdType": "Actual",
                    }
                },
                "timeGrain": "Monthly",
                "timePeriod": {"endDate": "2018-10-31T00:00:00Z", "startDate": "2017-10-01T00:00:00Z"},
            },
        },
    )
    print(response)


# x-ms-original-file: specification/consumption/resource-manager/Microsoft.Consumption/stable/2021-10-01/examples/CreateOrUpdateBudget.json
if __name__ == "__main__":
    main()
